@@ -120,7 +120,7 @@ exp_mail_it(struct objcore *oc, uint8_t cmds)
 			VSC_C_main->exp_mailed++;
 		}
 		oc->exp_flags |= cmds | OC_EF_POSTED;
-		AZ(pthread_cond_signal(&exphdl->condvar));
+		PTOK(pthread_cond_signal(&exphdl->condvar));
 	}
 }
 
@@ -354,8 +354,8 @@ exp_expire(struct exp_priv *ep, vtim_real now)
 		assert(oc->timer_idx == VBH_NOIDX);
 
 		CHECK_OBJ_NOTNULL(oc->objhead, OBJHEAD_MAGIC);
-		VSLb(&ep->vsl, SLT_ExpKill, "EXP_Expired x=%u t=%.0f",
-		    ObjGetXID(ep->wrk, oc), EXP_Ttl(NULL, oc) - now);
+		VSLb(&ep->vsl, SLT_ExpKill, "EXP_Expired xid=%ju t=%.0f",
+		    VXID(ObjGetXID(ep->wrk, oc)), EXP_Ttl(NULL, oc) - now);
 		ObjSendEvent(ep->wrk, oc, OEV_EXPIRE);
 		(void)HSH_DerefObjCore(ep->wrk, &oc, 0);
 	}
@@ -445,7 +445,7 @@ EXP_Init(void)
 	AN(ep);
 
 	Lck_New(&ep->mtx, lck_exp);
-	AZ(pthread_cond_init(&ep->condvar, NULL));
+	PTOK(pthread_cond_init(&ep->condvar, NULL));
 	VSTAILQ_INIT(&ep->inbox);
 	WRK_BgThread(&pt, "cache-exp", exp_thread, ep);
 	ep->thread = pt;
@@ -460,11 +460,11 @@ EXP_Shutdown(void)
 
 	Lck_Lock(&ep->mtx);
 	exp_shutdown = 1;
-	AZ(pthread_cond_signal(&ep->condvar));
+	PTOK(pthread_cond_signal(&ep->condvar));
 	Lck_Unlock(&ep->mtx);
 
 	AN(ep->thread);
-	AZ(pthread_join(ep->thread, &status));
+	PTOK(pthread_join(ep->thread, &status));
 	AZ(status);
 	memset(&ep->thread, 0, sizeof ep->thread);
 
