@@ -205,6 +205,15 @@ shard__assert(void)
 	assert(t2a == t2b);
 }
 
+static void v_matchproto_(vdi_release_f)
+vmod_shard_release(VCL_BACKEND dir)
+{
+	struct sharddir *shardd;
+
+	CAST_OBJ_NOTNULL(shardd, dir->priv, SHARDDIR_MAGIC);
+	sharddir_release(shardd);
+}
+
 static void v_matchproto_(vdi_destroy_f)
 vmod_shard_destroy(VCL_BACKEND dir)
 {
@@ -219,6 +228,7 @@ static const struct vdi_methods vmod_shard_methods[1] = {{
 	.type =		"shard",
 	.resolve =	vmod_shard_resolve,
 	.healthy =	vmod_shard_healthy,
+	.release =	vmod_shard_release,
 	.destroy =	vmod_shard_destroy,
 	.list =		vmod_shard_list
 }};
@@ -373,8 +383,6 @@ static inline uint32_t
 shard_get_key(VRT_CTX, const struct vmod_directors_shard_param *p)
 {
 	struct http *http;
-	struct strands s[1];
-	const char *sp[1];
 	VCL_ENUM by = default_by(p->by);
 
 	if (by == VENUM(KEY) || by == VENUM(BLOB))
@@ -390,10 +398,7 @@ shard_get_key(VRT_CTX, const struct vmod_directors_shard_param *p)
 			AN(ctx->http_bereq);
 			AN(http = ctx->http_bereq);
 		}
-		sp[0] = http->hd[HTTP_HDR_URL].b;
-		s->n = 1;
-		s->p = sp;
-		return (VRT_HashStrands32(s));
+		return (VRT_HashStrands32(TOSTRAND(http->hd[HTTP_HDR_URL].b)));
 	}
 	WRONG("by enum");
 }
@@ -929,13 +934,10 @@ shard_param_task_l(VRT_CTX, const void *id, const char *who,
 		return (p);
 	}
 
-	p = WS_Alloc(ctx->ws, sizeof *p);
-	if (p == NULL) {
-		shard_fail(ctx, who, "%s", "WS_Alloc failed");
+	WS_TASK_ALLOC_OBJ(ctx, p, VMOD_SHARD_SHARD_PARAM_MAGIC);
+	if (p == NULL)
 		return (NULL);
-	}
 	task->priv = p;
-	INIT_OBJ(p, VMOD_SHARD_SHARD_PARAM_MAGIC);
 	p->vcl_name = who;
 	p->scope = SCOPE_TASK;
 
